@@ -31,6 +31,8 @@
 
 (require 'magit)
 
+(declare-function ediff-quit "ediff-util" (reverse-default-keep-variants))
+
 ;;; Find Blob
 
 (define-obsolete-variable-alias 'magit-find-file-hook
@@ -42,6 +44,8 @@
 (defvar magit-find-blob-hook nil)
 (add-hook 'magit-find-blob-hook #'magit-blob-mode)
 
+(defvar-local magit-buffer--volatile nil)
+
 ;;;###autoload
 (defun magit-find-file (rev file)
   "View FILE from REV.
@@ -84,7 +88,7 @@ the line and column corresponding to that location."
           (magit-read-file-from-rev (if (member rev pseudo-revs) "HEAD" rev)
                                     prompt))))
 
-(defun magit-find-file-noselect (rev file &optional no-restore-position)
+(defun magit-find-file-noselect (rev file &optional no-restore-position volatile)
   "Read FILE from REV into a buffer and return the buffer.
 REV is a revision or one of \"{worktree}\" or \"{index}\"."
   (let* ((rev (pcase rev
@@ -112,7 +116,7 @@ REV is a revision or one of \"{worktree}\" or \"{index}\"."
              (unless (file-in-directory-p file topdir)
                (error "%s is not inside Git repository %s" file topdir))
              (with-current-buffer
-                 (magit-get-revision-buffer-create rev file-relative)
+                 (magit--get-blob-buffer rev file-relative volatile)
                (setq magit-buffer-revision rev)
                (setq magit-buffer-file-name file)
                (setq default-directory
@@ -128,12 +132,26 @@ REV is a revision or one of \"{worktree}\" or \"{index}\"."
           (apply #'magit-find-file--restore-position pos))))
     buffer))
 
-(defun magit-get-revision-buffer-create (rev file)
-  (magit-get-revision-buffer rev file t))
+(defun magit--get-blob-buffer (rev file &optional volatile)
+  ;; REV is assummed to be abbreviated and FILE to be relative.
+  (cond-let
+    ([buf (magit--find-buffer 'magit-buffer-revision rev
+                              'magit-buffer-file-name file)]
+     (with-current-buffer buf
+       (when (and (not volatile) magit-buffer--volatile)
+         (setq magit-buffer--volatile nil)
+         (rename-buffer (magit--blob-buffer-name rev file))))
+     buf)
+    ([buf (get-buffer-create (magit--blob-buffer-name rev file volatile))]
+     (with-current-buffer buf
+       (setq magit-buffer--volatile volatile))
+     buf)))
 
-(defun magit-get-revision-buffer (rev file &optional create)
-  (funcall (if create #'get-buffer-create #'get-buffer)
-           (format "%s.~%s~" file (subst-char-in-string ?/ ?_ rev))))
+(defun magit--blob-buffer-name (rev file &optional volatile)
+  (format "%s%s.~%s~"
+          (if volatile " " "")
+          file
+          (subst-char-in-string ?/ ?_ rev)))
 
 (defun magit--revert-blob-buffer (_ignore-auto _noconfirm)
   (let ((pos (magit-find-file--position)))
@@ -374,18 +392,32 @@ Currently this only adds the following key bindings.
   :package-version '(magit . "2.3.0"))
 
 (defun magit-bury-buffer (&optional kill-buffer)
-  "Bury the current buffer, or with a prefix argument kill it."
+  "Bury the current buffer, or with a prefix argument kill it.
+
+If the buffer is used by an Ediff session, refuse to kill or bury just
+that buffer.  That former would break the session and the latter makes
+little sense in this context.  Instead offer to quit the whole session."
   (interactive "P")
-  (if kill-buffer (kill-buffer) (bury-buffer)))
+  (cond ((bound-and-true-p ediff-this-buffer-ediff-sessions)
+         (ediff-quit nil))
+        (kill-buffer (kill-buffer))
+        ((bury-buffer))))
 
 (defun magit-bury-or-kill-buffer (&optional bury-buffer)
   "Bury the current buffer if displayed in multiple windows, else kill it.
-With a prefix argument only bury the buffer even if it is only displayed
-in a single window."
+
+With a prefix argument only bury the buffer even if it is only
+displayed in a single window.
+
+If the buffer is used by an Ediff session, refuse to kill or bury just
+that buffer.  That former would break the session and the latter makes
+little sense in this context.  Instead offer to quit the whole session."
   (interactive "P")
-  (if (or bury-buffer (cdr (get-buffer-window-list nil nil t)))
-      (bury-buffer)
-    (kill-buffer)))
+  (cond ((bound-and-true-p ediff-this-buffer-ediff-sessions)
+         (ediff-quit nil))
+        ((or bury-buffer (cdr (get-buffer-window-list nil nil t)))
+         (bury-buffer))
+        ((kill-buffer))))
 
 (defun magit-kill-this-buffer ()
   "Kill the current buffer."
